from django.contrib.admin import site
from django.core.exceptions import ImproperlyConfigured
from django.template import Library
from django.utils.module_loading import import_string

from ..conf import settings
from ..models import AppGroup

register = Library()


@register.simple_tag(takes_context=True)
def dashboard_app_list(context):
    try:
        request = context["request"]
    except KeyError:
        raise ImproperlyConfigured(
            "Django admin index requires 'django.template.context_processors.request' to be configured."
        )

    # Get the new app_list.
    app_list = AppGroup.objects.as_list(
        request, settings.show_remaining_apps(request.user.is_superuser)
    )

    # Use default app_list if there were no groups and no "misc" section.
    if not app_list:
        app_list = site.get_app_list(request)

    return app_list


@register.simple_tag()
def admin_index_settings():
    return settings.as_dict()


@register.simple_tag
def display_dropdown_menu(request):
    func = import_string(settings.DISPLAY_DROP_DOWN_MENU_CONDITION_FUNCTION)
    return func(request)



from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext_lazy as _


def validate_translation_json_format(value):
    if not isinstance(value, dict):
        raise ValidationError(
            _("The format of translations needs to be a JSON-object.")
        )

    language_codes = [item[0] for item in settings.LANGUAGES]
    for key, val in value.items():
        if key not in language_codes:
            raise ValidationError(
                _("The language code '{language_code}' is not enabled.").format(
                    language_code=key
                )
            )

        if not isinstance(val, str):
            raise ValidationError(
                _(
                    "The translation for language '{language_code}' is not a string."
                ).format(language_code=key)
            )


class TranslationsMixin(models.Model):
    translations = models.JSONField(
        _("translations"),
        default=dict,
        help_text=_(
            'A JSON-object that uses the Django language code as key and the localized name as value. If no translation can be found for the active language, the name is used as fallback. Example: {"en": "File", "nl": "Bestand"}'
        ),
        validators=[validate_translation_json_format],
    )

    class Meta:
        abstract = True



# Generated by Django 3.0.8 on 2020-07-24 15:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("admin_index", "0002_auto_20170802_1754"),
    ]

    operations = [
        migrations.AlterField(
            model_name="appgroup",
            name="order",
            field=models.PositiveIntegerField(
                db_index=True, editable=False, verbose_name="order"
            ),
        ),
        migrations.AlterField(
            model_name="applink",
            name="order",
            field=models.PositiveIntegerField(
                db_index=True, editable=False, verbose_name="order"
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-07-02 08:40
import django.contrib.contenttypes.models
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="AppGroup",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("order", models.PositiveIntegerField(db_index=True, editable=False)),
                ("name", models.CharField(max_length=200, verbose_name="name")),
                ("slug", models.SlugField(unique=True, verbose_name="slug")),
            ],
            options={
                "verbose_name_plural": "application groups",
                "verbose_name": "application group",
                "abstract": False,
                "ordering": ("order",),
            },
        ),
        migrations.CreateModel(
            name="AppLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("order", models.PositiveIntegerField(db_index=True, editable=False)),
                ("name", models.CharField(max_length=200)),
                ("link", models.CharField(max_length=200)),
                (
                    "app_group",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="admin_index.AppGroup",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "application links",
                "verbose_name": "application link",
                "abstract": False,
                "ordering": ("order",),
            },
        ),
        migrations.CreateModel(
            name="ContentTypeProxy",
            fields=[],
            options={
                "proxy": True,
                "ordering": ("app_label", "model"),
            },
            bases=("contenttypes.contenttype",),
            managers=[
                ("objects", django.contrib.contenttypes.models.ContentTypeManager()),
            ],
        ),
        migrations.AddField(
            model_name="appgroup",
            name="models",
            field=models.ManyToManyField(blank=True, to="admin_index.ContentTypeProxy"),
        ),
    ]



from django.conf import settings as django_settings


class Settings:
    @property
    def SHOW_REMAINING_APPS(self):
        return getattr(django_settings, "ADMIN_INDEX_SHOW_REMAINING_APPS", False)

    @property
    def SHOW_REMAINING_APPS_TO_SUPERUSERS(self):
        return getattr(
            django_settings, "ADMIN_INDEX_SHOW_REMAINING_APPS_TO_SUPERUSERS", True
        )

    def show_remaining_apps(self, is_superuser):
        if self.SHOW_REMAINING_APPS:
            return True

        return settings.SHOW_REMAINING_APPS_TO_SUPERUSERS and is_superuser

    @property
    def AUTO_CREATE_APP_GROUP(self):
        return getattr(django_settings, "ADMIN_INDEX_AUTO_CREATE_APP_GROUP", False)

    @property
    def SHOW_MENU(self):
        return getattr(django_settings, "ADMIN_INDEX_SHOW_MENU", True)

    @property
    def HIDE_APP_INDEX_PAGES(self):
        return getattr(django_settings, "ADMIN_INDEX_HIDE_APP_INDEX_PAGES", True)

    def as_dict(self):
        """
        Returns a `dict` with all settings.

        :return: A `dict` with settings.
        """
        return {k: getattr(self, k) for k in dir(self) if k.upper() == k}

    @property
    def DISPLAY_DROP_DOWN_MENU_CONDITION_FUNCTION(self):
        return getattr(
            django_settings,
            "ADMIN_INDEX_DISPLAY_DROP_DOWN_MENU_CONDITION_FUNCTION",
            "django_admin_index.utils.should_display_dropdown_menu",
        )


settings = Settings()



# Generated by Django 3.2.18 on 2023-05-03 17:10

from django.db import migrations, models

import django_admin_index.translations


class Migration(migrations.Migration):
    dependencies = [
        ("admin_index", "0004_auto_20230503_0723"),
    ]

    operations = [
        migrations.AddField(
            model_name="appgroup",
            name="translations",
            field=models.JSONField(
                default=dict,
                help_text='A JSON-object that uses the Django language code as key and the localized name as value. If no translation can be found for the active language, the name is used as fallback. Example: {"en": "File", "nl": "Bestand"}',
                validators=[
                    django_admin_index.translations.validate_translation_json_format
                ],
                verbose_name="translations",
            ),
        ),
        migrations.AddField(
            model_name="applink",
            name="translations",
            field=models.JSONField(
                default=dict,
                help_text='A JSON-object that uses the Django language code as key and the localized name as value. If no translation can be found for the active language, the name is used as fallback. Example: {"en": "File", "nl": "Bestand"}',
                validators=[
                    django_admin_index.translations.validate_translation_json_format
                ],
                verbose_name="translations",
            ),
        ),
    ]



# Generated by Django 3.2.18 on 2023-05-03 17:10

from django.conf import settings
from django.db import migrations


def copy_fallback_name_to_translations(apps, _):
    AppGroup = apps.get_model("admin_index", "AppGroup")
    AppLink = apps.get_model("admin_index", "AppLink")

    if not settings.LANGUAGE_CODE:
        return

    for app_group in AppGroup.objects.all():
        app_group.translations = {settings.LANGUAGE_CODE: app_group.name}
        app_group.save()

    for app_link in AppLink.objects.all():
        app_link.translations = {settings.LANGUAGE_CODE: app_link.name}
        app_link.save()


class Migration(migrations.Migration):
    dependencies = [
        ("admin_index", "0005_auto_20230503_1910"),
    ]

    operations = [
        migrations.RunPython(
            copy_fallback_name_to_translations, migrations.RunPython.noop
        ),
    ]
